# -*- coding: utf-8 -*-
# vim: ts=4 nowrap textwidth=80
# PHPdoc Plugin
# Copyright © 2011 Peter Bakota
#

from gi.repository import GObject, Gedit, Gtk, Gdk
from datetime import datetime
import re,os

class PHPDocPlugin(GObject.Object, Gedit.WindowActivatable):
    __gtype_name = "PHPDocPlugin"
    window = GObject.property(type=Gedit.Window)
    handler_ids = []

    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        view = self.window.get_active_view()
        if view:
            self.setup_phpdoc(view,'none')
        #print "Window %s activated" % self.window
    
    def do_deactivate(self):
        #print "Window %s deactivated" % self.window

        for handler_id,view in self.handler_ids:
            view.disconnect(handler_id)            

    def do_update_state(self):
        #print "Window %s updates its state" % self.window

        view = self.window.get_active_view()
        lang = 'none'
        if view:
            buf = view.get_buffer()
            language = buf.get_language()
            if language:
                lang = language.get_id()
            self.setup_phpdoc(view,lang)

    def setup_phpdoc(self,view,lang):
        if getattr(view, 'phpdoc_instance', False) == False:
            setattr(view, 'phpdoc_instance', PHPDoc())
            handler_id = view.connect('key-press-event',
                view.phpdoc_instance.key_press_handler)
            self.handler_ids.append((handler_id,view))
        view.phpdoc_instance.set_language(lang)

class PHPDoc:

    def __init__(self):
        self.__not_available = True
        self.re_trigger = None
        self.re_comment = None
        self.re_var = None
        self.re_class = None
        self.re_function = None
        self.re_module_start = None
        return

    def set_language(self, lang):
        #print "Lang: ",lang
        self.__not_available = False
        if lang == 'none':
            self.__not_available = True
        elif lang == 'php':
            self.re_trigger = re.compile(r'^\s*/\*\*$'); # PHPDoc trigger
            self.re_comment = re.compile(r'^\s*\*\s*(?!\/)'); # PHP comment trigger
            self.re_class = re.compile(r'^\s*(?P<DEFINED>[a-zA-Z]*)\s*(interface|class)\s*(?P<CLASSNAME>[^ ]+)\s*(extends)?\s*(?P<EXTENDS>[a-zA-Z0-9]+)?\s*(implements*)? *(?P<IMPLEMENTS>[a-zA-Z0-9_ ,]+)?.*$') # Class
            self.re_function = re.compile(r'^\s*(?P<SCOPE>[a-zA-Z ]*)function\s+(?P<FUNCNAME>[^ (]+)\s*\(\s*(?P<PARAMS>.*)\s*\)\s*[{;]?\s*$') # Function
            self.re_var = re.compile(r'^\s*(?P<SCOPE>[a-zA-Z ]*)\$(?P<VARNAME>[^ ;=]+)[ =]*(?P<VALUE>.*);?$') # Var
            self.re_module_start = re.compile(r'^\s*\<\?php$')
        else:
            self.__not_available = True

    def __get_current_line(self, view, buf):
        cursor_iter = buf.get_iter_at_mark(buf.get_insert())
        line_start_iter = cursor_iter.copy()
        view.backward_display_line_start(line_start_iter)
        return buf.get_text(line_start_iter, cursor_iter, True)

    def __get_bellow_line(self, view, buf):
        line_start_iter = buf.get_iter_at_mark(buf.get_insert())
        line_start_iter.forward_line()
        line_start_iter.set_line_offset(0)
        line_end_iter = line_start_iter.copy()
        line_end_iter.forward_line()
        line_end_iter.backward_char()
        return buf.get_text(line_start_iter, line_end_iter, True)

    def __get_above_line(self, view, buf):
        line_start_iter = buf.get_iter_at_mark(buf.get_insert())
        line_start_iter.backward_line()
        line_start_iter.set_line_offset(0)
        line_end_iter = buf.get_iter_at_mark(buf.get_insert())
        line_end_iter.set_line_offset(0)
        line_end_iter.backward_char()
        return buf.get_text(line_start_iter, line_end_iter, True)

    def key_press_handler(self, view, event):
        if (self.__not_available) or (event.keyval != Gdk.KEY_Return): # Key trigger
            return
        buf = view.get_buffer()
        s = self.__get_current_line(view, buf)
        if self.re_trigger.match(s):
            bellow = self.__get_bellow_line(view, buf)
            preceding_white_space_pattern = re.compile(r'^(\s*)')
            groups = preceding_white_space_pattern.search(bellow).groups()
            preceding_white_space = groups[0]
            m = self.re_class.match(bellow)
            if m:
                #print "Class name: (%s) %s extends: %s implements: %s" % (m.group('DEFINED'), m.group('CLASSNAME'), m.group('EXTENDS'), m.group('IMPLEMENTS'))
                buf.insert_at_cursor('\n')
                buf.insert_at_cursor(preceding_white_space)
                buf.insert_at_cursor(' * %s\n' % m.group('CLASSNAME'))
                buf.insert_at_cursor(preceding_white_space)
                buf.insert_at_cursor(' *\n')
                if m.group('DEFINED') !='':
                    buf.insert_at_cursor(preceding_white_space)
                    buf.insert_at_cursor(' * @%s\n' % m.group('DEFINED'))
                buf.insert_at_cursor(preceding_white_space)
                buf.insert_at_cursor(' */')
            else:
                m = self.re_function.match(bellow)
                if m:
                    #print "Function: (%s) %s params: %s" % (m.group("SCOPE"), m.group("FUNCNAME"), m.group("PARAMS"))
                    buf.insert_at_cursor('\n')
                    buf.insert_at_cursor(preceding_white_space)
                    buf.insert_at_cursor(' * %s\n' % m.group("FUNCNAME"))
                    buf.insert_at_cursor(preceding_white_space)
                    buf.insert_at_cursor(' *\n')
                    #buf.insert_at_cursor(preceding_white_space)
                    #buf.insert_at_cursor(' * @access %s\n' % m.group("SCOPE"))
                    if m.group("SCOPE").find("static")!=-1:
                        buf.insert_at_cursor(preceding_white_space)
                        buf.insert_at_cursor(' * @static\n')
                    for param in m.group("PARAMS").split(","):
                        param = param.strip()
                        if param == '':
                            continue
                        try:
                            (type, name) = param.split(" ")
                        except ValueError:
                            type = "unknown"
                            name = param
                        buf.insert_at_cursor(preceding_white_space)
                        buf.insert_at_cursor(' * @param %s %s\n' % (type,name))
                    buf.insert_at_cursor(preceding_white_space)
                    buf.insert_at_cursor(' * @return void\n')
                    buf.insert_at_cursor(preceding_white_space)
                    buf.insert_at_cursor(' */')
                else:
                    m = self.re_var.match(bellow)
                    if m:
                        #print "Var: (%s) %s value: %s" % (m.group("SCOPE"), m.group("VARNAME"), m.group("VALUE"))
                        buf.insert_at_cursor('\n')
                        if m.group("SCOPE").find("static")!=-1:
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' * @static\n')
                        #buf.insert_at_cursor(preceding_white_space)
                        #buf.insert_at_cursor(' * @access %s\n' % m.group("SCOPE"))
                        buf.insert_at_cursor(preceding_white_space)
                        #buf.insert_at_cursor(' * @var $%s\n' % m.group("VARNAME"))
                        #buf.insert_at_cursor(preceding_white_space)
                        buf.insert_at_cursor(' * @var unknown\n')
                        buf.insert_at_cursor(preceding_white_space)
                        buf.insert_at_cursor(' */')
                    else:
                        lineno = buf.get_iter_at_mark(buf.get_insert()).get_line()
                        if lineno == 0:
                            return
                        above = self.__get_above_line(view, buf)
                        #print "Above line is: [%s]" % above
                        if self.re_module_start.match(above):
                            buf.insert_at_cursor('\n')
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' * %s\n' % buf.get_short_name_for_display())
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' *\n')
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' * @author %s\n' % os.environ['USER'])
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' * @version $Id$\n') # subversion ID property
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' * @since %s\n' % datetime.today().strftime('%c')) # see: http://docs.python.org/library/datetime.html
                            buf.insert_at_cursor(preceding_white_space)
                            buf.insert_at_cursor(' */')
                        else:
                            return
            return True
        elif self.re_comment.match(s):
            # Continue in comment mode
            buf.insert_at_cursor('\n' + s[:s.find('*')] + '* ')
            return True


        
